package es.verdnatura.presentation.view.feature.precontrol

import android.content.Context
import android.os.Build
import androidx.annotation.RequiresApi
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.SalixCallback
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.view.feature.collection.mapper.map
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import es.verdnatura.presentation.view.feature.sacador.model.TicketStateList
import es.verdnatura.presentation.view.feature.sacador.model.TicketStateSalix
import retrofit2.Response

class ShowTicketViewModel(val context: Context) : BaseViewModel(context) {

    private val _collectionTicketList by lazy { MutableLiveData<CollectionVO>() }
    val collectionTicketList: LiveData<CollectionVO> = _collectionTicketList

    private val _lastTicketList by lazy { MutableLiveData<TicketStateList>() }
    val lastTicketList: LiveData<TicketStateList> = _lastTicketList

    fun collectionTicketGet(collectionFk: Int, sectorFk: Int, print: String, type: String) {
        //Tarea 6276
        //salix.collectionGetTickets(collectionFk,sectorFk,print.toInt(),type)
        silex.collection_getTickets(collectionFk, sectorFk, print, type)
            .enqueue(object :
                SilexCallback<CollectionVO>(context) {
                override fun onError(t: Throwable) {
                    _collectionTicketList.value = CollectionVO(
                        0,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<CollectionVO>) {
                    if (response.body() != null) {
                        _collectionTicketList.value = response.body()?.let { it.map(context) }
                    } else {
                        _collectionTicketList.value = CollectionVO(
                            0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }
                }
            })
    }

    fun myLastTickets(userId: Int) {
        salix.myLastModified("""{"where": {"userFk": $userId}""")
            .enqueue(object : SalixCallback<List<TicketStateSalix>>(context) {
                @RequiresApi(Build.VERSION_CODES.O)
                override fun onSuccess(response: Response<List<TicketStateSalix>>) {
                    _lastTicketList.value = response.body()?.let {  TicketStateList(it.sortedByDescending { it.created }) }
                }

            })
    }
}